@@ -4,11 +4,12 @@ import logging
 import xml.etree.ElementTree as ET
 from collections import namedtuple
 from pathlib import Path
+from textwrap import indent
 
 from mako.exceptions import RichTraceback
-from mako.runtime import Context
 from mako.template import Template
 from pkg_resources import DistributionNotFound, require
+from ruamel.yaml import YAML
 
 TraceableInfo = namedtuple("TraceableInfo", ['matrix_prefix', 'type', 'header_prefix'])
 UTEST = TraceableInfo('UTEST_', 'unit', '_unit_test_report_')
@@ -17,11 +18,12 @@ QTEST = TraceableInfo('QTEST_', 'qualification', '_qualification_test_report_')
 TEMPLATE_FILE = Path(__file__).parent.joinpath('xunit2rst.mako')
 
 
-def render_template(destination, **kwargs):
+def render_template(destination, only="", **kwargs):
     """ Renders the Mako template, and writes output file to the specified destination.
 
     Args:
         destination (Path): Location of the output file.
+        only (str): Expression for 'only' directive, which will only be added when this string is not empty.
         **kwargs (dict): Variables to be used in the Mako template.
 
     Raises:
@@ -29,18 +31,22 @@ def render_template(destination, **kwargs):
         Exception: Re-raised Exception coming from Mako template.
     """
     destination.parent.mkdir(parents=True, exist_ok=True)
-    with open(str(destination), 'w', newline='\n', encoding='utf-8') as rst_file:
-        template = Template(filename=str(TEMPLATE_FILE), output_encoding='utf-8', input_encoding='utf-8')
-        try:
-            template.render_context(Context(rst_file, **kwargs))
-        except Exception as exc:
-            traceback = RichTraceback()
-            logging.error("Exception raised in Mako template, which will be re-raised after logging line info:")
-            logging.error("File %s, line %s, in %s: %r", *traceback.traceback[-1])
-            raise exc
+    template = Template(filename=str(TEMPLATE_FILE))
+    try:
+        rst_content = template.render(**kwargs)
+    except Exception as exc:
+        traceback = RichTraceback()
+        logging.error("Exception raised in Mako template, which will be re-raised after logging line info:")
+        logging.error("File %s, line %s, in %s: %r", *traceback.traceback[-1])
+        raise exc
+    if only:
+        rst_content = f".. only:: {only}\n\n{indent(rst_content, ' ' * 4)}"
+    with open(str(destination), 'w', encoding='utf-8', newline='\n') as rst_file:
+        rst_file.write(rst_content)
 
 
-def generate_xunit_to_rst(input_file, rst_file, itemize_suites, failure_message, log_file, add_links, *prefix_args):
+def generate_xunit_to_rst(input_file, rst_file, itemize_suites, failure_message, log_file, add_links, *prefix_args,
+                          **kwargs):
     """ Processes input arguments, calls mako template function while passing all needed parameters.
 
     Args:
@@ -51,7 +57,7 @@ def generate_xunit_to_rst(input_file, rst_file, itemize_suites, failure_message,
         log_file (str): Optional path to the HTML log file, empty when not specified.
         add_links (bool): True to add links to the HTML log file for each test case
     """
-    test_suites, prefix_set = parse_xunit_root(input_file)
+    test_suites, prefix_set, report_info_file = parse_xunit_root(input_file)
 
     prefix_set, prefix = build_prefix_and_set(test_suites, prefix_set, *prefix_args)
 
@@ -59,6 +65,13 @@ def generate_xunit_to_rst(input_file, rst_file, itemize_suites, failure_message,
     if report_name.endswith('_report'):
         report_name = report_name[:-len('_report')]
 
+    extra_content_map = {}
+    if report_info_file:
+        yaml = YAML(typ='safe', pure=True)
+        if not report_info_file.is_absolute():
+            report_info_file = input_file.parent / report_info_file
+        extra_content_map = {name.lower(): content for name, content in yaml.load(report_info_file).items()}
+
     render_template(
         rst_file,
         test_suites=test_suites,
@@ -69,9 +82,26 @@ def generate_xunit_to_rst(input_file, rst_file, itemize_suites, failure_message,
         failure_message=failure_message,
         log_file=log_file,
         add_links=add_links,
+        extra_content_map=extra_content_map,
+        **kwargs,
     )
 
 
+def look_for_content_file(element):
+    """ If the element contains metadata, it returns the path to the content file, if configured.
+
+    Args:
+        element (xml.etree.ElementTree.Element): XML element to inspect
+
+    Returns:
+        Path/None: Path to the content file, if found in the given element; None otherwise
+    """
+    if element.tag in ('properties', 'traits'):
+        for prop in element:
+            if prop.attrib['name'].lower() == 'xunit2rst content file':
+                return Path(prop.attrib['value'])
+
+
 def parse_xunit_root(input_file):
     """
     This function parses the root element of the XML file and returns a testsuites root element and the set of prefixes
@@ -85,6 +115,7 @@ def parse_xunit_root(input_file):
     Returns:
         xml.etree.ElementTree.Element: Root element of the element tree with 'testsuites' as tag.
         TraceableInfo: Namedtuple holding the prefixes to use for building traceability output.
+        Path/None: Path to the content file; None if not configured
     """
     tree = ET.parse(str(input_file))
     root_input = tree.getroot()
@@ -96,15 +127,21 @@ def parse_xunit_root(input_file):
         test_suites = root_input
         prefix_set = UTEST
 
+    report_info_file = None
     for suite in test_suites:
         if suite.tag != 'testsuite':
+            value = look_for_content_file(suite)
+            if value:
+                report_info_file = value
             test_suites.remove(suite)
             continue
         for test in suite:
             if test.tag != 'testcase':
+                value = look_for_content_file(test)
+                if value:
+                    report_info_file = value
                 suite.remove(test)
-
-    return test_suites, prefix_set
+    return test_suites, prefix_set, report_info_file
 
 
 def build_prefix_and_set(test_suites, prefix_set, prefix, trim_suffix, type_):
@@ -160,7 +197,7 @@ def verify_prefix_set(prefix_set, prefix, type_):
         TraceableInfo: namedtuple that contains prefix and other info regarding the type of test
 
     Raises:
-        ValueError: The unit-or-integration argument is used, but is invalid.
+        ValueError: The --type argument is used, but is invalid.
     """
     type_map = {
         'u': UTEST,
@@ -198,6 +235,9 @@ def create_parser():
                             dest='rst_output_file',
                             type=Path,
                             help='The output RST file')
+    arg_parser.add_argument("--only", dest="expression", default="",
+                            help="Expression of tags for Sphinx' `only` directive that surrounds all RST content. "
+                            "By default, no `only` directive is generated.")
     arg_parser.add_argument('-s', '--itemize-suites',
                             action='store_true',
                             help="Flag to itemize testsuite elements instead of testcase elements.")
@@ -243,6 +283,7 @@ def main():
         args.prefix,
         args.trim_suffix,
         args.type,
+        only=args.expression,
     )
 
 
